package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.ClienteCorporativo;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEmpresa;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPadrao;

@Controller("busquedaClienteCorporativoController")
@Scope("prototype")
public class BusquedaClienteCorporativoController extends MyGenericForwardComposer {

	private static Logger log = LogManager.getLogger(BusquedaClienteCorporativoController.class); 
	private static final long serialVersionUID = 1L;
	@Autowired	
	private EmpresaService empresaService;
	@Autowired
	private transient PagedListWrapper<ClienteCorporativo> plwpagingClienteCorporativo;
	private MyListbox clienteCorporativoList;
	private Paging pagingClienteCorporativo;
	private Textbox txtNome;
	private Textbox txtIdentificacao;
	private MyComboboxEmpresa cmbEmpresa;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		clienteCorporativoList.setItemRenderer(new RenderPadrao<ClienteCorporativo>(ClienteCorporativo.class));
		clienteCorporativoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				ClienteCorporativo cc = (ClienteCorporativo) clienteCorporativoList.getSelected();
				verClienteCorporativo(cc);
			}
		});

		refreshLista();

		txtNome.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verClienteCorporativo(ClienteCorporativo cc) {
		if (cc == null) {
			return;
		}
		
		clienteCorporativoList.getAnnotatedProperties().stream().findFirst();
		
		Map args = new HashMap();
		args.put("clienteCorporativo", cc);
		args.put("clienteCorporativoList", clienteCorporativoList);

		openWindow("/gui/configuraciones_comerciales/negcorporativos/editarClienteCorporativo.zul",
				Labels.getLabel("editarClienteCorporativoController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<ClienteCorporativo> configCategoriaBusqueda =new HibernateSearchObject<ClienteCorporativo>(ClienteCorporativo.class,pagingClienteCorporativo.getPageSize());

		if (txtNome.getText().length() > 0) {
			configCategoriaBusqueda.addFilterLike("nomeClienteCorp", "%" + txtNome.getText().trim().concat("%"));
		}
		
		if (txtIdentificacao.getText().length() > 0) {
			configCategoriaBusqueda.addFilterEqual("identificacao", Long.valueOf(txtIdentificacao.getText()));
		}
		
		Comboitem empresa = cmbEmpresa.getSelectedItem();
		if( empresa != null ) {
			configCategoriaBusqueda.addFilterEqual("empresaId", ((Empresa)empresa.getValue()).getEmpresaId() );
		}

		configCategoriaBusqueda.addFilterEqual("activo", Boolean.TRUE);
		configCategoriaBusqueda.addSortAsc("nomeClienteCorp");	

		plwpagingClienteCorporativo.init(configCategoriaBusqueda, clienteCorporativoList, pagingClienteCorporativo);

		if (clienteCorporativoList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaClienteCorporativoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
				log.error(ex);				
			}
		}else {
			preencheNomeEmpresaClientes();
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verClienteCorporativo(new ClienteCorporativo());
	}
	
	private void preencheNomeEmpresaClientes() {
		for (Object item : clienteCorporativoList.getListData()) {
			ClienteCorporativo cli = (ClienteCorporativo)item;
			if( StringUtils.isEmpty(cli.getNombEmpresa()) ) {
				cli.setNombEmpresa( empresaService.buscarNomeEmpresa( cli.getEmpresaId()) );
			}
		}		
	}
}
